Module hash

Module hash 

FxHash for fast and non-cryptographic hashing

Type Aliases

HashMap
Type alias for a hash map that uses the Fx hashing algorithm.
HashSet
Type alias for a hash set that uses the Fx hashing algorithm.