IntermediateState

Struct IntermediateState 

pub struct IntermediateState<TermClass, NonTerm> {
    pub shift_goto_map_term: Vec<(TermClass, ShiftTarget<usize>)>,
    pub shift_goto_map_nonterm: Vec<(NonTerm, ShiftTarget<usize>)>,
    pub reduce_map: Vec<(TermClass, Vec<usize>)>,
    pub ruleset: Vec<LR0ItemRef>,
}
One decoded parser-table row used by generated code and the grammar builder before converting to a concrete runtime table layout.

Fields

shift_goto_map_term: Vec<(TermClass, ShiftTarget<usize>)>shift_goto_map_nonterm: Vec<(NonTerm, ShiftTarget<usize>)>reduce_map: Vec<(TermClass, Vec<usize>)>ruleset: Vec<LR0ItemRef>